@@ -9,7 +9,7 @@ use Carp;
 use List::Util qw(first);
 use WebService::Linode::Base;
 
-our $VERSION = '0.20';
+our $VERSION = '0.21';
 our @ISA     = ("WebService::Linode::Base");
 our $AUTOLOAD;
 
@@ -28,6 +28,7 @@ my %validation = (
         distributions => [ [], [ 'distributionid' ] ],
         kernels => [ [], [ 'isxen', 'kernelid' ] ],
         linodeplans => [ [], [ 'planid' ] ],
+        nodebalancers => [ [], [] ],
         stackscripts => [ [], [qw( distributionid distributionvendor keywords )] ],
     },
     domain => {
@@ -62,9 +63,9 @@ my %validation = (
         update => [ [ 'configid' ], [qw( comments devtmpfs_automount disklist helper_depmod helper_disableupdatedb helper_xen kernelid label linodeid ramlimit rootdevicecustom rootdevicenum rootdevicero runlevel )] ],
     },
     linode_disk => {
-        create => [ [qw( label linodeid size type )], [] ],
+        create => [ [qw( label linodeid size type )], [qw( fromdistributionid isreadonly rootpass rootsshkey )] ],
         createfromdistribution => [ [qw( distributionid label linodeid rootpass size )], [ 'rootsshkey' ] ],
-        createfromstackscript => [ [qw( distributionid label linodeid rootpass size stackscriptid stackscriptudfresponses )], [] ],
+        createfromstackscript => [ [qw( distributionid label linodeid rootpass size stackscriptid stackscriptudfresponses )], [ 'rootsshkey' ] ],
         delete => [ [ 'diskid', 'linodeid' ], [] ],
         duplicate => [ [ 'diskid', 'linodeid' ], [] ],
         list => [ [ 'linodeid' ], [ 'diskid' ] ],
@@ -75,13 +76,14 @@ my %validation = (
         addprivate => [ [ 'linodeid' ], [] ],
         addpublic => [ [ 'linodeid' ], [] ],
         list => [ [ 'linodeid' ], [ 'ipaddressid' ] ],
+        setrdns => [ [ 'hostname', 'ipaddressid' ], [] ],
         swap => [ [ 'ipaddressid' ], [ 'tolinodeid', 'withipaddressid' ] ],
     },
     linode_job => {
         list => [ [ 'linodeid' ], [ 'jobid', 'pendingonly' ] ],
     },
     nodebalancer => {
-        create => [ [ 'datacenterid', 'paymentterm' ], [ 'clientconnthrottle', 'label' ] ],
+        create => [ [ 'datacenterid' ], [qw( clientconnthrottle label paymentterm )] ],
         delete => [ [ 'nodebalancerid' ], [] ],
         list => [ [], [ 'nodebalancerid' ] ],
         update => [ [ 'nodebalancerid' ], [ 'clientconnthrottle', 'label' ] ],
@@ -179,19 +181,7 @@ L<WebService::Linode::Base>.
 
 =head1 Methods from the Linode API
 
-=head3 avail_stackscripts
-
-Optional Parameters:
-
-=over 4
-
-=item * distributionid
-
-=item * distributionvendor
-
-=item * keywords
-
-=back
+=head3 avail_datacenters
 
 =head3 avail_kernels
 
@@ -215,7 +205,21 @@ Optional Parameters:
 
 =back
 
-=head3 avail_datacenters
+=head3 avail_nodebalancers
+
+=head3 avail_stackscripts
+
+Optional Parameters:
+
+=over 4
+
+=item * distributionid
+
+=item * distributionvendor
+
+=item * keywords
+
+=back
 
 =head3 avail_distributions
 
@@ -227,6 +231,16 @@ Optional Parameters:
 
 =back
 
+=head3 domain_delete
+
+Required Parameters:
+
+=over 4
+
+=item * domainid
+
+=back
+
 =head3 domain_create
 
 Required Parameters:
@@ -265,16 +279,6 @@ Optional Parameters:
 
 =back
 
-=head3 domain_delete
-
-Required Parameters:
-
-=over 4
-
-=item * domainid
-
-=back
-
 =head3 domain_update
 
 Required Parameters:
@@ -325,6 +329,18 @@ Optional Parameters:
 
 =back
 
+=head3 domain_resource_delete
+
+Required Parameters:
+
+=over 4
+
+=item * domainid
+
+=item * resourceid
+
+=back
+
 =head3 domain_resource_create
 
 Required Parameters:
@@ -357,7 +373,7 @@ Optional Parameters:
 
 =back
 
-=head3 domain_resource_delete
+=head3 domain_resource_list
 
 Required Parameters:
 
@@ -365,6 +381,12 @@ Required Parameters:
 
 =item * domainid
 
+=back
+
+Optional Parameters:
+
+=over 4
+
 =item * resourceid
 
 =back
@@ -401,72 +423,6 @@ Optional Parameters:
 
 =back
 
-=head3 domain_resource_list
-
-Required Parameters:
-
-=over 4
-
-=item * domainid
-
-=back
-
-Optional Parameters:
-
-=over 4
-
-=item * resourceid
-
-=back
-
-=head3 linode_mutate
-
-Required Parameters:
-
-=over 4
-
-=item * linodeid
-
-=back
-
-=head3 linode_create
-
-Required Parameters:
-
-=over 4
-
-=item * datacenterid
-
-=item * planid
-
-=back
-
-Optional Parameters:
-
-=over 4
-
-=item * paymentterm
-
-=back
-
-=head3 linode_reboot
-
-Required Parameters:
-
-=over 4
-
-=item * linodeid
-
-=back
-
-Optional Parameters:
-
-=over 4
-
-=item * configid
-
-=back
-
 =head3 linode_webconsoletoken
 
 Required Parameters:
@@ -477,68 +433,6 @@ Required Parameters:
 
 =back
 
-=head3 linode_boot
-
-Required Parameters:
-
-=over 4
-
-=item * linodeid
-
-=back
-
-Optional Parameters:
-
-=over 4
-
-=item * configid
-
-=back
-
-=head3 linode_resize
-
-Required Parameters:
-
-=over 4
-
-=item * linodeid
-
-=item * planid
-
-=back
-
-=head3 linode_clone
-
-Required Parameters:
-
-=over 4
-
-=item * datacenterid
-
-=item * linodeid
-
-=item * planid
-
-=back
-
-Optional Parameters:
-
-=over 4
-
-=item * paymentterm
-
-=back
-
-=head3 linode_shutdown
-
-Required Parameters:
-
-=over 4
-
-=item * linodeid
-
-=back
-
 =head3 linode_delete
 
 Required Parameters:
@@ -557,6 +451,54 @@ Optional Parameters:
 
 =back
 
+=head3 linode_boot
+
+Required Parameters:
+
+=over 4
+
+=item * linodeid
+
+=back
+
+Optional Parameters:
+
+=over 4
+
+=item * configid
+
+=back
+
+=head3 linode_create
+
+Required Parameters:
+
+=over 4
+
+=item * datacenterid
+
+=item * planid
+
+=back
+
+Optional Parameters:
+
+=over 4
+
+=item * paymentterm
+
+=back
+
+=head3 linode_shutdown
+
+Required Parameters:
+
+=over 4
+
+=item * linodeid
+
+=back
+
 =head3 linode_update
 
 Required Parameters:
@@ -611,6 +553,50 @@ Optional Parameters:
 
 =back
 
+=head3 linode_resize
+
+Required Parameters:
+
+=over 4
+
+=item * linodeid
+
+=item * planid
+
+=back
+
+=head3 linode_mutate
+
+Required Parameters:
+
+=over 4
+
+=item * linodeid
+
+=back
+
+=head3 linode_clone
+
+Required Parameters:
+
+=over 4
+
+=item * datacenterid
+
+=item * linodeid
+
+=item * planid
+
+=back
+
+Optional Parameters:
+
+=over 4
+
+=item * paymentterm
+
+=back
+
 =head3 linode_list
 
 Optional Parameters:
@@ -621,6 +607,36 @@ Optional Parameters:
 
 =back
 
+=head3 linode_reboot
+
+Required Parameters:
+
+=over 4
+
+=item * linodeid
+
+=back
+
+Optional Parameters:
+
+=over 4
+
+=item * configid
+
+=back
+
+=head3 linode_config_delete
+
+Required Parameters:
+
+=over 4
+
+=item * configid
+
+=item * linodeid
+
+=back
+
 =head3 linode_config_create
 
 Required Parameters:
@@ -663,18 +679,6 @@ Optional Parameters:
 
 =back
 
-=head3 linode_config_delete
-
-Required Parameters:
-
-=over 4
-
-=item * configid
-
-=item * linodeid
-
-=back
-
 =head3 linode_config_update
 
 Required Parameters:
@@ -737,42 +741,16 @@ Optional Parameters:
 
 =back
 
-=head3 linode_disk_create
+=head3 linode_disk_duplicate
 
 Required Parameters:
 
 =over 4
 
-=item * label
+=item * diskid
 
 =item * linodeid
 
-=item * size
-
-=item * type
-
-=back
-
-=head3 linode_disk_createfromstackscript
-
-Required Parameters:
-
-=over 4
-
-=item * distributionid
-
-=item * label
-
-=item * linodeid
-
-=item * rootpass
-
-=item * size
-
-=item * stackscriptid
-
-=item * stackscriptudfresponses
-
 =back
 
 =head3 linode_disk_createfromdistribution
@@ -801,29 +779,51 @@ Optional Parameters:
 
 =back
 
-=head3 linode_disk_resize
+=head3 linode_disk_list
 
 Required Parameters:
 
 =over 4
 
+=item * linodeid
+
+=back
+
+Optional Parameters:
+
+=over 4
+
 =item * diskid
 
+=back
+
+=head3 linode_disk_create
+
+Required Parameters:
+
+=over 4
+
+=item * label
+
 =item * linodeid
 
 =item * size
 
+=item * type
+
 =back
 
-=head3 linode_disk_duplicate
-
-Required Parameters:
+Optional Parameters:
 
 =over 4
 
-=item * diskid
+=item * fromdistributionid
 
-=item * linodeid
+=item * isreadonly
+
+=item * rootpass
+
+=item * rootsshkey
 
 =back
 
@@ -839,6 +839,50 @@ Required Parameters:
 
 =back
 
+=head3 linode_disk_createfromstackscript
+
+Required Parameters:
+
+=over 4
+
+=item * distributionid
+
+=item * label
+
+=item * linodeid
+
+=item * rootpass
+
+=item * size
+
+=item * stackscriptid
+
+=item * stackscriptudfresponses
+
+=back
+
+Optional Parameters:
+
+=over 4
+
+=item * rootsshkey
+
+=back
+
+=head3 linode_disk_resize
+
+Required Parameters:
+
+=over 4
+
+=item * diskid
+
+=item * linodeid
+
+=item * size
+
+=back
+
 =head3 linode_disk_update
 
 Required Parameters:
@@ -861,7 +905,7 @@ Optional Parameters:
 
 =back
 
-=head3 linode_disk_list
+=head3 linode_ip_addpublic
 
 Required Parameters:
 
@@ -871,34 +915,18 @@ Required Parameters:
 
 =back
 
-Optional Parameters:
-
-=over 4
-
-=item * diskid
-
-=back
-
-=head3 linode_ip_swap
+=head3 linode_ip_setrdns
 
 Required Parameters:
 
 =over 4
 
+=item * hostname
+
 =item * ipaddressid
 
 =back
 
-Optional Parameters:
-
-=over 4
-
-=item * tolinodeid
-
-=item * withipaddressid
-
-=back
-
 =head3 linode_ip_addprivate
 
 Required Parameters:
@@ -909,16 +937,6 @@ Required Parameters:
 
 =back
 
-=head3 linode_ip_addpublic
-
-Required Parameters:
-
-=over 4
-
-=item * linodeid
-
-=back
-
 =head3 linode_ip_list
 
 Required Parameters:
@@ -937,6 +955,26 @@ Optional Parameters:
 
 =back
 
+=head3 linode_ip_swap
+
+Required Parameters:
+
+=over 4
+
+=item * ipaddressid
+
+=back
+
+Optional Parameters:
+
+=over 4
+
+=item * tolinodeid
+
+=item * withipaddressid
+
+=back
+
 =head3 linode_job_list
 
 Required Parameters:
@@ -957,42 +995,6 @@ Optional Parameters:
 
 =back
 
-=head3 stackscript_create
-
-Required Parameters:
-
-=over 4
-
-=item * distributionidlist
-
-=item * label
-
-=item * script
-
-=back
-
-Optional Parameters:
-
-=over 4
-
-=item * description
-
-=item * ispublic
-
-=item * rev_note
-
-=back
-
-=head3 stackscript_delete
-
-Required Parameters:
-
-=over 4
-
-=item * stackscriptid
-
-=back
-
 =head3 stackscript_update
 
 Required Parameters:
@@ -1031,13 +1033,17 @@ Optional Parameters:
 
 =back
 
-=head3 nodebalancer_config_create
+=head3 stackscript_create
 
 Required Parameters:
 
 =over 4
 
-=item * nodebalancerid
+=item * distributionidlist
+
+=item * label
+
+=item * script
 
 =back
 
@@ -1045,29 +1051,21 @@ Optional Parameters:
 
 =over 4
 
-=item * algorithm
+=item * description
 
-=item * check
+=item * ispublic
 
-=item * check_attempts
+=item * rev_note
 
-=item * check_body
+=back
 
-=item * check_interval
+=head3 stackscript_delete
 
-=item * check_path
+Required Parameters:
 
-=item * check_timeout
+=over 4
 
-=item * port
-
-=item * protocol
-
-=item * ssl_cert
-
-=item * ssl_key
-
-=item * stickiness
+=item * stackscriptid
 
 =back
 
@@ -1083,13 +1081,13 @@ Required Parameters:
 
 =back
 
-=head3 nodebalancer_config_update
+=head3 nodebalancer_config_create
 
 Required Parameters:
 
 =over 4
 
-=item * configid
+=item * nodebalancerid
 
 =back
 
@@ -1141,6 +1139,56 @@ Optional Parameters:
 
 =back
 
+=head3 nodebalancer_config_update
+
+Required Parameters:
+
+=over 4
+
+=item * configid
+
+=back
+
+Optional Parameters:
+
+=over 4
+
+=item * algorithm
+
+=item * check
+
+=item * check_attempts
+
+=item * check_body
+
+=item * check_interval
+
+=item * check_path
+
+=item * check_timeout
+
+=item * port
+
+=item * protocol
+
+=item * ssl_cert
+
+=item * ssl_key
+
+=item * stickiness
+
+=back
+
+=head3 nodebalancer_node_delete
+
+Required Parameters:
+
+=over 4
+
+=item * nodeid
+
+=back
+
 =head3 nodebalancer_node_create
 
 Required Parameters:
@@ -1165,16 +1213,6 @@ Optional Parameters:
 
 =back
 
-=head3 nodebalancer_node_delete
-
-Required Parameters:
-
-=over 4
-
-=item * nodeid
-
-=back
-
 =head3 nodebalancer_node_update
 
 Required Parameters:
